###
#
# Listen is the legal property of mehdi abaakouk <theli48@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
###

import gobject
import time
import gst
import sys

#gst.use_threads(True)

class GstPlayer(gobject.GObject):
    __gsignals__ = {
        "play-end" : (gobject.SIGNAL_RUN_LAST,
                gobject.TYPE_NONE,
                (gobject.TYPE_PYOBJECT,)),
        "new-media" : (gobject.SIGNAL_RUN_LAST,
                gobject.TYPE_NONE,
                (gobject.TYPE_PYOBJECT,))
    }

    def __init__(self,player):
        gobject.GObject.__init__(self)

        try: import gconf
        except ImportError: sink_name = "alsasink"
        else:
            c = gconf.client_get_default()
            val = c.get("/system/gstreamer/0.8/default/audiosink")
            if val.type == gconf.VALUE_STRING: sink_name = val.get_string()
            else: sink_name = "alsasink"

        try: sink = gst.parse_launch(sink_name)
        except gobject.GError, err:
            if sink_name != "osssink":
                print "%r failed, falling back to osssink (%s)." % (sink_name, err)
                try: sink = gst.parse_launch("osssink")
                except gobject.GError: sink = None
                else: sink_name = "osssink"

        if sink==None:
            print "No audio sink found for Gstreamer"
            sys.exit(1)
        #print "use \"",sink_name,"\" audio sink for gstreamer"

        self.bin = gst.element_factory_make('playbin')
        self.bin.set_property('video-sink', None)
        self.bin.set_property('audio-sink',sink)
        self.bin.connect_object('eos', self.__end,None)
        #self.bin.connect_object('error', self.__error,None)
        self.bin.connect('found-tag', self.__tag)
        self.bin.set_state(gst.STATE_NULL)
        self.paused = True
        self.player = player

    def __notify_source(self,object,spec,uri):
        source = self.bin.get_property('source')
        source.set_property("device",uri[uri.find("#")+1:])
        self.bin.disconnect(self.handler_notify_id)


    #TAG wrapper for iRadop
    def __tag(self, pipeline, source, taglist):
        if isinstance(taglist,gst.TagList) and self.player.current_info!=None and self.player.current_info.is_stream==True:
            for k in taglist.keys():
                value = str(taglist[k])#.strip()
                #print k," - ",value
                if not value: continue
                #Set album on current stream with for fancy wiev
                if self.player.current_info.get_property("artist")==None:
                    self.player.current_info.set_property("artist",self.player.current_info.get_property("title"))
                if k == "title":
                    self.player.current_info.set_property("title",value)
                if k == "comment":
                    self.player.current_info.set_property("album",value)
            gobject.idle_add(self.player.on_playlist_changed,self.player.current_info)
        pass

    def __error(self,*param):
        self.paused = True
        gobject.idle_add(self.player.playlist_next_cb)
        print "error"

    def __end(self,param):
        self.paused = True

        gobject.idle_add(self.player.playlist_next_cb)
        #print "stop"

    def get_state(self):
        return self.bin.get_state()

    def play(self):
        if (self.bin.set_state(gst.STATE_PLAYING)==gst.STATE_SUCCESS):
            self.paused = False
        else:
            self.paused = True
            gobject.idle_add(self.player.playlist_next_cb)

    def pause(self):
        self.bin.set_state(gst.STATE_PAUSED)
        self.paused = True

    def stop(self):
        self.bin.set_state(gst.STATE_NULL)
        self.paused = True

    def play_new(self,song):
        #print "Play: "+uri
        uri = song.get_property("uri")
        if uri.find("cdda://")==0:
            self.bin.set_property('uri',uri[:uri.find("#")])
            self.handler_notify_id = self.bin.connect("notify::source",self.__notify_source,uri)
        else:
            self.bin.set_property('uri',uri)
        self.emit("new-media",song)
        self.bin.set_state(gst.STATE_NULL)
        self.play();

    def is_playable(self):
        if self.bin.get_property('uri'):
            return True
        else :
            return False

    def set_volume(self, v):
        self.__volume = v
        self.bin.set_property('volume', v)
        """
        if self.song is None: self.bin.set_property('volume', v)
        else: self.bin.set_property('volume', v * self.song.replay_gain())
        """

    def get_volume(self): return self.__volume
    volume = property(get_volume, set_volume)


    def get_position(self):
        if self.bin.get_state() != gst.STATE_NULL and self.bin.get_property('uri'):
            p = self.bin.query(gst.QUERY_POSITION, gst.FORMAT_TIME)
            p //= gst.MSECOND
            return p
        else: return 0

    def seek(self, pos):
        if self.bin.get_property('uri'):
            pos = max(0, int(pos))

            total = self.bin.query(gst.QUERY_TOTAL, gst.FORMAT_TIME)
            ms = pos * gst.MSECOND
            if total<=ms:
                self.paused = True
                gobject.idle_add(self.player.playlist_next_cb)
            else:
                event = gst.event_new_seek(
                    gst.FORMAT_TIME|gst.SEEK_METHOD_SET|gst.SEEK_FLAG_FLUSH, ms)
                self.bin.send_event(event)
            #self.info.seek(self.song, pos)



