###
#
# Listen is the legal property of mehdi abaakouk <theli48@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
###


import gtk
import xml.sax.saxutils

from context import *
from view_browser import *
from view_playlist import *
from view_ipod import *
from view_iradio import *
from view_audioconvert import *

from misc_widget import WindowBase
from misc_widget import WindowWarning


class ListenMediaSource:
    def __init__(self,widget,media_organizer):
        self.media_organizer = media_organizer
        self.widget = widget
        self.widget.show_all()
        self.widget.set_no_show_all(True)
        self.widget.hide()
        self.show_quick_search = True
        self.image = gtk.gdk.pixbuf_new_from_file(config.PIXMAP_DIR+"/biblio.png")

        #listen.py file launch run() for each source loaded
        #and begin with high priority, and finish with low priority
        self.startup_priority = 0
        self.menu = self.get_menu()

    def get_elements_source(self):
        info = {}
        info["label"] = "No label"
        info["label_yalign"] = 0.8
        info["arg_element"]={}
        info["image"] = self.image
        info["stock-id"] = None
        info["stock-size"] = None
        info["image_height"] = -1
        info["image_yalign"] = 0.5
        return [info]

    def populate(self, arg_element):
        pass

    def get_widget(self):
        return self.widget

    def run(self):
        pass

    def save_config(self):
        pass

    def on_drag_song_received(self,arg_element,song):
        return False

    def on_drag_data_get(self,selection,arg_element):
        return False

    def get_menu(self):
        return None

    def on_key_press(self, widget, event):
        pass

    def on_activate(self,arg_element):
        pass


class ListenMLBrowserSource(ListenMediaSource):
    def __init__(self,media_organizer):
        widget = ListenUIBrowserView(media_organizer)
        ListenMediaSource.__init__(self,widget,media_organizer)
        self.image = gtk.gdk.pixbuf_new_from_file(config.PIXMAP_DIR+"/biblio.png")
        self.startup_priority = 10

    def get_elements_source(self):
        info =  ListenMediaSource.get_elements_source(self)[0]
        info["label"] = "<span size=\"large\"><b>"+xml.sax.saxutils.escape(_("Library"))+"</b></span>"
        info["label_yalign"] = 1
        #info["image"] = self.image
        info["image_height"] = 30
        info["image_yalign"] = 1
        return [info]

    def run(self):
        DB = ListenDBManager()
        self.widget.refresh_cache(DB)
        gobject.idle_add(self.widget.populate_view)

        #Start a new thread to get missing cover
        #self.start_thread_download_cover()

    def start_thread_download_cover(self):
        threading.Thread(target=self.thread_download_cover).start()

    def thread_download_cover(self):
        #Fecthing album cover in background
        for song in self.widget.songs_cache:
            gobject.id_add(song.get_album_art)

        #10 min
        REFRESH_COVER = 100*60*10
        gobject.timeout_add(REFRESH_COVER, self.start_thread_download_cover)

    def save_config(self):
        select_artist = ""
        sep = ""
        for key,value in self.widget.last_select_artist.iteritems():
            select_artist += sep+key
            sep = "<#>"
        select_album = ""
        sep = ""
        for key,value in self.widget.last_select_album.iteritems():
            select_album += sep+key
            sep = "<#>"

        config.set("setting","last_selected_artist",select_artist)
        config.set("setting","last_selected_album",select_album)
        config.set("setting","pos_browser","%d"%self.widget.get_position())





class ListenContextSource(ListenMediaSource):
    def __init__(self,media_organizer):
        widget = ListenContext(media_organizer)
        ListenMediaSource.__init__(self,widget,media_organizer)
        #self.image = gtk.gdk.pixbuf_new_from_file(config.PIXMAP_DIR+"/wikipedia.png")
        self.show_quick_search = False

    def get_elements_source(self):
        info =  ListenMediaSource.get_elements_source(self)[0]
        info["stock-id"] = gtk.STOCK_INFO
        info["stock-size"] = gtk.ICON_SIZE_BUTTON
        info["image_height"] = 25
        info["image"] = None
        info["label"] = "<span size=\"large\"><b>"+xml.sax.saxutils.escape(_("Context"))+"</b></span>"
        info["image_yalign"] = 0.95
        return [info]

    def run(self):
        gobject.idle_add(self.widget.home_widget.run)
        #gobject.idle_add(self.widget.wikipedia_widget.run)
        #gobject.idle_add(self.widget.lyrics_widget.run)

"""
class ListenAudioConvertSource(ListenMediaSource):
    def __init__(self,media_organizer):
        widget = ListenUIAudioConvert(media_organizer)
        ListenMediaSource.__init__(self,widget,media_organizer)
        #self.image = gtk.gdk.pixbuf_new_from_file(config.PIXMAP_DIR+"/wikipedia.png")
        self.show_quick_search = False

    def get_elements_source(self):
        info =  ListenMediaSource.get_elements_source(self)[0]
        info["stock-id"] = gtk.STOCK_REFRESH
        info["stock-size"] = gtk.ICON_SIZE_DND
        info["image_height"] = 30
        info["image"] = None
        info["label"] = "<span size=\"large\"><b>"+xml.sax.saxutils.escape(_("Audio converter"))+"</b></span>"
        info["image_yalign"] = 1
        return [info]


    def on_drag_song_received(self,arg_element,list_song):
        for song in list_song:
            self.widget.convert_view.add_song(song)
"""


class ListenLocalPlaylistSource(ListenMediaSource):
    def __init__(self,media_organizer):
        self.view = ListenUITreeviewSongPlaylist(media_organizer)
        widget =  get_scrolled_window(self.view)
        ListenMediaSource.__init__(self,widget,media_organizer)
        self.image_search = gtk.gdk.pixbuf_new_from_file(config.PIXMAP_DIR+"/chercher.png")
        self.image_playlist = gtk.gdk.pixbuf_new_from_file(config.PIXMAP_DIR+"/playlist.png")

        self.current_playlist = None

    def get_elements_source(self):

        playlists = DBManager.get_playlists()
        list_element = []
        for playlist in playlists:
            if playlist["name"]=="__autosaved__":
                continue
            info =  ListenMediaSource.get_elements_source(self)[0]
            info["arg_element"] = {"id":playlist["id"]}

            if playlist["name"]=="__searchresult__":
                info["label"] = xml.sax.saxutils.escape(_("Search result"))
                info["image"] = self.image_search
                list_element.insert(0,info)
            else:
                info["label"] = xml.sax.saxutils.escape(playlist["name"])
                info["image"] = self.image_playlist
                list_element.append(info)
        return list_element

    def populate(self,arg_element):
        self.current_playlist = arg_element["id"]
        self.view.populate(arg_element["id"])


    def on_drag_song_received(self,arg_element,list_song):
        for song in list_song:
            DBManager.add_song_to_playlist(song,arg_element["id"])
        if self.current_playlist == arg_element["id"]:
            self.view.populate(arg_element["id"])



    def on_drag_data_get(self,selection,arg_element):
        list_uri = []
        list_song = []
        for row in self.view.model_song:
            song = row[0]
            list_song.append(song)
            uri = song.get_property("uri")
            uri = utils.convert_to_uri(uri)
            list_uri.append(uri)

        self.media_organizer.get_toplevel().last_drag_drop = list_song
        selection.set_uris(list_uri)


    def on_key_press(self, widget, event):
        if event.keyval == gtk.gdk.keyval_from_name("Delete"):
            self.remove_selected()
            return True

    def on_activate(self,arg_element):
        self.play_selected(None)

    def export_selected(self,btn=None):
        saver = gtk.FileChooserDialog(_("Export playlist"),
                                           None,
                                           gtk.FILE_CHOOSER_ACTION_SAVE,
                                           None)
        saver.add_button(gtk.STOCK_SAVE, gtk.RESPONSE_OK)
        saver.add_button(gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE)

        saver.set_current_folder(os.path.expanduser("~"))
        saver.set_current_name("*.m3u")

        ffilter = gtk.FileFilter()
        ffilter.set_name(_("M3U files"))
        ffilter.add_mime_type("audio/mpegurl")
        ffilter.add_mime_type("audio/x-mpegurl")
        ffilter.add_mime_type("audio/m3u")
        ffilter.add_pattern("*.m3u")
        saver.add_filter(ffilter)
        response = saver.run()
        if response ==  gtk.RESPONSE_OK:
            list_song = list ( item[0] for item in self.view.model_song)
            utils.export_playlist(saver.get_filename(),list_song)
        saver.destroy()



    def remove_selected(self,btn=None):
        DBManager.delete_playlist(self.current_playlist)
        #Repopulate mediaorganizer
        self.media_organizer.populate()


    def play_selected(self,btn):
        while gtk.events_pending():gtk.main_iteration()
        play = True
        for row in self.view.model_song:
            self.media_organizer.player.playlist.add_song(row[0],False,play)
            play = False

    def enqueue_selected(self,btn):
        for row in self.view.model_song:
            self.media_organizer.player.playlist.add_song(row[0],False,False)


    def rename_selected(self,btn=None):
        info = DBManager.get_playlists("id=%d"%self.current_playlist)
        info = info[0]

        dialog = WindowBase(_("Rename playlist"),config.PIXMAP_DIR+"/playlist_big.png")
        hbox = gtk.HBox(0,6)
        hbox.pack_start(gtk.Label(_("Name")+" :"))
        self.entry= gtk.Entry()
        self.entry.set_activates_default(True)
        if info["name"]=="__searchresult__":
            name="playlist"
        else:
            name = info["name"]
        self.entry.set_text(name)
        self.entry.grab_focus()
        hbox.pack_start(self.entry)
        dialog.add_widget(hbox)
        dialog.show_all()
        result = dialog.run()
        if result == gtk.RESPONSE_ACCEPT:
            text = self.entry.get_text()
            if text!="":
                DBManager.rename_playlist(self.current_playlist,text)
                self.media_organizer.populate()

        dialog.destroy()



    def get_menu(self):
        menu_ui = """
        <ui>
            <popup name="PlaylistMenu">
              <menuitem action="Play"/>
              <menuitem action="Queue"/>
              <separator/>
              <menuitem action="Rename"/>
              <menuitem action="Delete"/>
              <separator/>
              <menuitem action="Export"/>
            </popup>
        </ui>
        """
        self.uimanager = gtk.UIManager()
        self.uimanager.add_ui_from_string(menu_ui)
        accelgroup = self.uimanager.get_accel_group()

        actiongroup = gtk.ActionGroup('Listen')
        actiongroup.add_actions([('Play', gtk.STOCK_MEDIA_PLAY, _('_Play'), None,
                                         _('Play this playlist'), self.play_selected)])
        actiongroup.add_actions([('Queue', gtk.STOCK_ADD, _('_Queue'), None,
                                         _('Queue this playlist'), self.enqueue_selected)])

        actiongroup.add_actions([('Rename', gtk.STOCK_EDIT, _('_Rename'), None,
                                         _('Rename this playlist'), self.rename_selected)])

        actiongroup.add_actions([('Delete', gtk.STOCK_DELETE, _('_Delete'), None,
                          _('Delete this playlist'), self.remove_selected)])

        actiongroup.add_actions([('Export', gtk.STOCK_SAVE_AS, _('_Export'), None,
                                         _('Export this playlist'), self.export_selected)])


        self.uimanager.insert_action_group(actiongroup, 1)
        #gobject.idle_add(self.get_toplevel().add_accel_group,accelgroup)

        return self.uimanager.get_widget("/PlaylistMenu")


class ListenIRadioSource(ListenMediaSource):
    def __init__(self,media_organizer):
        self.view = ListenUITreeviewSongIRadio(media_organizer)
        widget = get_scrolled_window(self.view)
        ListenMediaSource.__init__(self,widget,media_organizer)
        #self.image = gtk.gdk.pixbuf_new_from_file(config.PIXMAP_DIR+"/wikipedia.png")
        self.show_quick_search = False

    def get_elements_source(self):
        info =  ListenMediaSource.get_elements_source(self)[0]
        info["stock-id"] = stock.PLAYLIST_RADIO
        info["stock-size"] = gtk.ICON_SIZE_BUTTON
        info["image"] = None
        info["image_height"] = 30
        info["image_yalign"] = 1
        info["label"] = "<span size=\"large\"><b>"+xml.sax.saxutils.escape(_("iRadio"))+"</b></span>"
        return [info]


    def populate(self,arg_element):
        self.view.populate_view()

class ListenIPodSource(ListenMediaSource):
    def __init__(self,media_organizer,ipod,device_udi):
        self.device_udi = device_udi
        widget = ListenUIIPodBrowserView(media_organizer,ipod)
        ListenMediaSource.__init__(self,widget,media_organizer)
        self.image = gtk.gdk.pixbuf_new_from_file(config.PIXMAP_DIR+"/ipod.png")
        self.startup_priority = 9
        self.ipod = ipod

    def get_elements_source(self):
        info =  ListenMediaSource.get_elements_source(self)[0]
        info["label"] = "<span size=\"large\"><b>"+xml.sax.saxutils.escape(self.ipod.get_ipod_name())+"</b></span>"
        info["label_yalign"] = 1
        #info["image"] = self.image
        info["image_height"] = 30
        info["image_yalign"] = 1
        return [info]

    def run(self):
        gobject.idle_add(self.widget.ipod.refresh_cache)
        if self.widget.get_property("visible"):
            gobject.idle_add(self.widget.populate_view)

    def populate(self,arg_element):
        self.widget.populate_view()

    def on_drag_song_received(self,arg_element,list_song):

        if not self.media_organizer.ipod_transfert_manager.ipod_update_cover:
            for song in list_song:
                self.media_organizer.ipod_transfert_manager.add_transfert_queue(song,self.ipod,None)
        else:
            WindowWarning(_("<b>A other work in progress on ipod</b>")+"\n"+\
                           _("Wait it finish and retry"))




    def new_playlist(self,btn=None):
        dialog = WindowBase(_("New iPod playlist"),config.PIXMAP_DIR+"/playlist_big.png")
        hbox = gtk.HBox(0,6)
        hbox.pack_start(gtk.Label(_("Name")+" :"))
        self.entry= gtk.Entry()
        self.entry.set_activates_default(True)
        self.entry.grab_focus()
        hbox.pack_start(self.entry)
        dialog.add_widget(hbox)
        dialog.show_all()
        result = dialog.run()
        if result == gtk.RESPONSE_ACCEPT:
            text = self.entry.get_text()
            if text!="":
                self.ipod.create_playlist(text)
                self.media_organizer.populate()

        dialog.destroy()

    def rename_ipod(self,btn=None):
        dialog = WindowBase(_("Rename iPod"),config.PIXMAP_DIR+"/ipod_big.png")
        hbox = gtk.HBox(0,6)
        hbox.pack_start(gtk.Label(_("Name")+" :"))
        self.entry= gtk.Entry()
        self.entry.set_activates_default(True)

        self.entry.set_text(self.ipod.get_ipod_name())
        self.entry.grab_focus()
        hbox.pack_start(self.entry)
        dialog.add_widget(hbox)
        dialog.show_all()
        result = dialog.run()
        if result == gtk.RESPONSE_ACCEPT:
            text = self.entry.get_text()
            if text!="":
                self.ipod.set_ipod_name(text)
                self.media_organizer.populate()

        dialog.destroy()

    def update_cover(self,btn):
        if not self.media_organizer.ipod_transfert_manager.ipod_is_in_transfer:
            self.media_organizer.ipod_transfert_manager.start_refresh_cover(self.ipod)
        else:
            WindowWarning(_("<b>A other work in progress on ipod</b>")+"\n"+\
                       _("Wait it finish and retry"))



    def get_menu(self):
        menu_ui = """
            <ui>
                <popup name="IpodMenu">
                  <menuitem action="Rename"/>
                  <menuitem action="New"/>
                  <menuitem action="Cover"/>
                </popup>
            </ui>
            """

        #<menuitem action="Rename"/>
        self.uimanager = gtk.UIManager()
        self.uimanager.add_ui_from_string(menu_ui)
        accelgroup = self.uimanager.get_accel_group()

        actiongroup = gtk.ActionGroup('Listen')
        actiongroup.add_actions([('New', gtk.STOCK_NEW, _('New playlist'), None,
                                         _('New playlist'), self.new_playlist)])
        actiongroup.add_actions([('Rename', gtk.STOCK_EDIT, _('Rename iPod'), None,
                                         _('Rename iPod'), self.rename_ipod)])
        actiongroup.add_actions([('Cover', gtk.STOCK_REFRESH, _('Update Cover'), None,
                                         _('Update Cover'), self.update_cover)])



        self.uimanager.insert_action_group(actiongroup, 1)
        #gobject.idle_add(self.get_toplevel().add_accel_group,accelgroup)

        return self.uimanager.get_widget("/IpodMenu")

class ListenIPodPlaylistSource(ListenMediaSource):
    def __init__(self,media_organizer,ipod,device_udi):
        self.device_udi = device_udi
        self.view = ListenUIIPodPlaylistSongview(media_organizer,ipod)
        widget = get_scrolled_window(self.view)
        ListenMediaSource.__init__(self,widget,media_organizer)
        self.image = gtk.gdk.pixbuf_new_from_file(config.PIXMAP_DIR+"/playlist_ipod.png")
        self.startup_priority = 9
        self.ipod = ipod
        self.current_playlist = None

    def get_elements_source(self):
        list_playlist = self.ipod.get_all_playlist()
        list_item = []
        for playlist in list_playlist:
            info =  ListenMediaSource.get_elements_source(self)[0]
            info["label"] = xml.sax.saxutils.escape(playlist.name)
            info["arg_element"] = {"playlist":playlist}
            list_item.append(info)
        return list_item

    def run(self):
        if self.widget.get_property("visible") and self.current_playlist!=None:
            gobject.idle_add(self.view.populate(self.current_playlist))

    def populate(self,arg_element):
        self.current_playlist = arg_element["playlist"]
        self.view.populate(arg_element["playlist"])

    def on_drag_data_get(self,selection,arg_element):
        list_uri = []
        list_song = []
        for song in self.ipod.get_song_from_playlist(arg_element["playlist"]):
            list_song.append(song)
            uri = song.get_property("uri")
            uri = utils.convert_to_uri(uri)
            list_uri.append(uri)

        self.media_organizer.get_toplevel().last_drag_drop = list_song
        selection.set_uris(list_uri)

    def on_drag_song_received(self,arg_element,list_song):

        if not self.media_organizer.ipod_transfert_manager.ipod_update_cover:
            for song in list_song:
                self.media_organizer.ipod_transfert_manager.add_transfert_queue(song,self.ipod,arg_element["playlist"])
        else:
            WindowWarning(_("<b>A other work in progress on ipod</b>")+"\n"+\
                       _("Wait it finish and retry"))

    def rename_playlist(self,btn=None):
        dialog = WindowBase(_("Rename playlist"),config.PIXMAP_DIR+"/playlist_big.png")
        hbox = gtk.HBox(0,6)
        hbox.pack_start(gtk.Label(_("Name")+" :"))
        self.entry= gtk.Entry()
        self.entry.set_activates_default(True)

        self.entry.set_text(self.current_playlist.name)
        self.entry.grab_focus()
        hbox.pack_start(self.entry)
        dialog.add_widget(hbox)
        dialog.show_all()
        result = dialog.run()
        if result == gtk.RESPONSE_ACCEPT:
            text = self.entry.get_text()
            if text!="":
                self.ipod.rename_playlist(self.current_playlist,text)
                self.media_organizer.populate()

        dialog.destroy()

    def new_playlist(self,btn=None):
        dialog = WindowBase(_("New iPod playlist"),config.PIXMAP_DIR+"/playlist_big.png")
        hbox = gtk.HBox(0,6)
        hbox.pack_start(gtk.Label(_("Name")+" :"))
        self.entry= gtk.Entry()
        self.entry.set_activates_default(True)
        self.entry.grab_focus()
        hbox.pack_start(self.entry)
        dialog.add_widget(hbox)
        dialog.show_all()
        result = dialog.run()
        if result == gtk.RESPONSE_ACCEPT:
            text = self.entry.get_text()
            if text!="":
                self.ipod.create_playlist(text)
                self.media_organizer.populate()

        dialog.destroy()

    def remove_playlist(self,btn=None):
        self.ipod.remove_playlist(self.current_playlist)
        self.media_organizer.populate()

    def get_menu(self):
        menu_ui = """
            <ui>
                <popup name="IpodMenu">
                  <menuitem action="New"/>
                  <menuitem action="Rename"/>
                  <menuitem action="Delete"/>
                </popup>
            </ui>
            """

        self.uimanager = gtk.UIManager()
        self.uimanager.add_ui_from_string(menu_ui)
        accelgroup = self.uimanager.get_accel_group()

        actiongroup = gtk.ActionGroup('Listen')
        actiongroup.add_actions([('New', gtk.STOCK_NEW, _('New playlist'), None,
                                         _('New playlist'), self.new_playlist)])
        actiongroup.add_actions([('Rename', gtk.STOCK_EDIT, _('Rename playlist'), None,
                                         _('Rename this playlist'), self.rename_playlist)])
        actiongroup.add_actions([('Delete', gtk.STOCK_EDIT, _('Delete playlist'), None,
                                         _('Delete this playlist'), self.remove_playlist)])



        self.uimanager.insert_action_group(actiongroup, 1)
        #gobject.idle_add(self.get_toplevel().add_accel_group,accelgroup)

        return self.uimanager.get_widget("/IpodMenu")

