###
#
# Listen is the legal property of mehdi abaakouk <theli48@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
###

import gst
import gnomevfs
import utils
import gobject
import sys
import config
import os
from song import Song

#Configuration
mp3_quality = {
    "cbr": (64, 96, 128, 192, 256),
    "abr": (64, 96, 128, 192, 256),
    "vbr": (9, 7, 5, 3, 1),
}
mp3_mode = {
    "cbr" : (0,"bitrate"),
    "abr" : (3,"vbr-mean-bitrate"),
    "vbr" : (4,"vbr-quality")
}
ogg_quality =  (0,0.2,0.4,0.6,0.8)

FORMAT_PERCENT_SCALE = 10000
"""
Encode file in background
"""
class Transcoder(gobject.GObject):
    __gsignals__ = {
        "finish" : (gobject.SIGNAL_RUN_LAST,
                gobject.TYPE_NONE,
                (gobject.TYPE_STRING,))
    }
    def __init__(self,song=None,format="mp3",quality=3,mode="cbr"):
        gobject.GObject.__init__(self)

        extention="."+format
        self.song = song
        input_file = self.song.get_property("uri")

        self.output_file = config.TMP_PATH + utils.get_filename(input_file[:input_file.rindex(".")] + extention)

        if os.path.exists(self.output_file):
            os.unlink(self.output_file)
        # Add protocol for gnomevfs
        self.output_file = "file://"+self.output_file

        #Make the new song instance
        self.new_song = Song()
        self.new_song.read_from_song(self.song)
        #set as no db song
        self.new_song.id=None
        #change with future file
        self.new_song.set_property("uri",self.output_file)


        self.pipeline = gst.Pipeline()
        # Source
        filesrc = self.make_element("gnomevfssrc", "src")
        filesrc.set_property("location", input_file)
        self.add(filesrc)

        # Decoder
        decoder = self.make_element("spider", "spider")
        self.add(decoder)

        # Converter
        audioconverter = self.make_element("audioconvert", "audioconverter")
        self.add(audioconverter)

        #Encoder
        mp3enc = self.make_element("lame", "encoder")

        if format=="mp3":
            mp3enc.set_property("quality",2)
            mp3enc.set_property("xingheader","true")

            mp3enc.set_property("vbr", mp3_mode[mode][0])
            mp3enc.set_property(mp3_mode[mode][1], mp3_quality[mode][quality])
            self.add(mp3enc)

        if format=="ogg":
            vorbisenc = self.make_element("vorbisenc", "encoder")
            if quality is not None:
                    vorbisenc.set_property("quality", ogg_quality[quality] )
            self.add(vorbisenc)

        if format=="flac":
            self.add(self.make_element("flacenc", "encoder"))


        # Sink
        sink = self.make_element("gnomevfssink", "sink")
        sink.set_property("location", self.output_file)
        self.add(sink)

        #self.pipeline.connect_object('eos', self.__finish,None)

    def make_element(self, elementkind, elementname):
        factory = gst.element_factory_find(elementkind)
        if factory:
            return factory.create(elementname)
        else:
            return None

    def work(self):
        if self.pipeline.get_state() == gst.STATE_NULL:
            print " not start"
            self.stop()
            return False
        else:
            self.pipeline.iterate()
            return True


    def add(self, element):
        assert self.pipeline.get_state() != gst.STATE_PLAYING
        elements = self.pipeline.get_list()
        self.pipeline.add(element)
        if elements and not elements[-1].link(element):
            raise "No link"

    def is_finish(self):
        if self.pipeline.get_state() == gst.STATE_PLAYING:
            return False
        else:
            return True

    def __finish(self):
        self.emit("finish",self.output_file)
        self.stop()

    def get_progress(self):
        elements = self.pipeline.get_list()
        value = elements[0].query(gst.QUERY_POSITION, gst.FORMAT_PERCENT)
        return float(value) / float(FORMAT_PERCENT_SCALE)

    def start(self):
        self.pipeline.set_state(gst.STATE_PLAYING)
        self.id = gobject.idle_add(self.work)

    def stop(self):
        if 'id' in dir(self) and self.id is not None:
            gobject.source_remove(self.id)
            self.id = None

        #Write tags informations
        if self.is_finish():
            self.new_song.write_to_file()
            self.emit("finish",self.output_file)
        else:
            self.pipeline.set_state(gst.STATE_NULL)
        #del self.pipeline

    def get_new_song(self):
        return self.new_song
