###
#
# Listen is the legal property of mehdi abaakouk <theli48@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
###
import gtk
import gobject


from db_manager import *
from view_song import *
from transcode import Transcoder
from misc_widget import get_scrolled_window

class ListenUIAudioConvert(gtk.VBox):
    def __init__(self,media_organizer):
        gtk.VBox.__init__(self)
        self.media_organizer = media_organizer

        self.convert_view = ListenUITreeviewAudioConvert(media_organizer)

        self.btn_convert = gtk.Button(_("Convert"))
        self.btn_convert.set_image(gtk.image_new_from_stock(gtk.STOCK_REFRESH, gtk.ICON_SIZE_LARGE_TOOLBAR))
        self.btn_convert.connect("clicked",self.on_convert)
        self.btn_convert.show_all()
        self.btn_convert.set_no_show_all(True)

        self.btn_cancel = gtk.Button(stock=gtk.STOCK_CANCEL)
        self.btn_cancel.connect("clicked",self.on_cancel_convert)
        self.btn_cancel.set_no_show_all(True)

        self.btn_clean = gtk.Button(stock=gtk.STOCK_CLEAR)
        self.btn_clean.connect("clicked",self.on_clear_list)
        self.btn_clean.show_all()
        self.btn_clean.set_no_show_all(True)

        self.progress = gtk.ProgressBar()
        self.progress.set_no_show_all(True)

        btn_box = gtk.HBox(False,6)
        btn_box.pack_end(self.btn_cancel,False,False)
        btn_box.pack_end(self.btn_convert,False,False)
        btn_box.pack_start(self.btn_clean,False,False)
        btn_box.pack_start(self.progress,True,True)

        self.pack_start(get_scrolled_window(self.convert_view))
        self.pack_start(btn_box,False,False)
        self.set_spacing(6)

    def on_clear_list(self,btn=None):
        self.convert_view.model_song.clear()


    def on_cancel_convert(self,btn=None):
        self.btn_convert.show()
        self.btn_clean.show()
        self.btn_cancel.hide()
        self.progress.hide()
        pass

    def on_convert(self,btn=None):
        win = ListenUIWindowAudioConvert()
        res = win.run()

        if res == gtk.RESPONSE_ACCEPT:
            self.run_convertion()

        win.destroy()

    def run_convertion(self):
            self.btn_convert.hide()
            self.btn_clean.hide()
            self.btn_cancel.show()
            self.progress.show()

class ListenUIWindowAudioConvert(gtk.Dialog):
    def __init__(self):
        gtk.Dialog.__init__(self,_("Audio convert utility"),None,
                 gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                     (gtk.STOCK_OK, gtk.RESPONSE_ACCEPT,
                          gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
                 )

        self.list_format = ["mp3","ogg","flac"]
        self.select_format = self.make_select("format",self.list_format)

        self.list_quality = [_("Very low"),_("Low"),_("Normal"),_("High"),_("Very high")]
        self.select_quality = self.make_select("quality",self.list_quality)

        self.list_mode = [_("Constant")+" (CBR)",_("Average")+" (ABR)",_("Variable")+" (VBR)"]
        self.select_mode = self.make_select("mp3_mode",self.list_mode)

        self.l_format = gtk.Label(_("File format"))
        self.l_format.set_alignment(0,0.5)
        self.l_quality = gtk.Label(_("Quality"))
        self.l_quality.set_alignment(0,0.5)
        self.l_mode = gtk.Label(_("Bitrate"))
        self.l_mode.set_alignment(0,0.5)

        combo_box = gtk.HBox(False,6)
        combo_box.set_border_width(6)
        combo_box.pack_start(self.l_format,False,False)
        combo_box.pack_start(self.select_format,False,False)
        combo_box.pack_start(self.l_quality,False,False)
        combo_box.pack_start(self.select_quality,False,False)
        combo_box.pack_start(self.l_mode,False,False)
        combo_box.pack_start(self.select_mode,False,False)

        self.vbox.pack_start(combo_box,False,False)
        self.set_resizable(False)
        self.show_all()


    def make_select(self,name,list_item):
        combo = gtk.combo_box_new_text()
        combo.connect("changed",self.combo_change,name)
        for label in list_item:
            combo.append_text(label)
        return combo

    def combo_change(self,combo,name):
        if name=="format":
            self.update_quality()

    def update_quality(self):
        index = self.select_format.get_active()
        format = self.list_format[index]
        """
        i = 0
        for name,label in self.list_format.iteritems():
            if index==i:
                format = name
                break;
            i += 1
        """

class ListenUITreeviewAudioConvert(ListenUITreeviewSong):
    def __init__(self,media_organizer,allow_column={"title":True,"album":True,"artist":True,"track":True,"duration":True}):

        ListenUITreeviewSong.__init__(self,media_organizer,allow_column,False)

        self.menu = self.get_menu()


        self.media_organizer = media_organizer

        self.enable_model_drag_dest([("text/uri-list", 0, 0)], gtk.gdk.ACTION_COPY)
        self.connect("drag-data-received", self.on_drag_data_reveived)
        #self.connect("drag-data-get", self.on_drag_data_get)
        self.connect("key-press-event", self.on_key_press)

        self.set_model(self.model_song)


    def on_drag_data_reveived(self,treeview, context, x, y, selection, info, timestamp):
        drop_info = treeview.get_dest_row_at_pos(x, y)
        model = treeview.get_model()
        #print selection.data
        if selection.target=="text/uri-list":
            if context.get_source_widget()==self:
                position = gtk.TREE_VIEW_DROP_AFTER
                model, rows = treeview.get_selection().get_selected_rows()
                if len(rows) < 1:
                    return
                rows = [row[0] for row in rows]

                if drop_info:
                    new_pos, position = drop_info
                    new_pos = new_pos[0]
                else:
                    new_pos = len(model)-1

                if new_pos < rows[0]:
                    rows.reverse()

                i=0
                for row in rows:
                    row += i
                    #print position
                    #print row," to ",new_pos
                    iter = model.get_iter(row)
                    is_current_item = model.get_value(iter,1)
                    new_iter = model.get_iter(new_pos)
                    if position == gtk.TREE_VIEW_DROP_BEFORE or position == gtk.TREE_VIEW_DROP_INTO_OR_BEFORE:
                        model.move_before(iter, new_iter);
                    if position == gtk.TREE_VIEW_DROP_AFTER or position == gtk.TREE_VIEW_DROP_INTO_OR_AFTER:
                        model.move_after(iter, new_iter);
                    if new_pos < rows[0]:
                        i += 1
                    else:
                        i -= 1
            else :
                if drop_info:
                    new_pos, position = drop_info
                    new_pos = new_pos[0]
                    if position == gtk.TREE_VIEW_DROP_AFTER:
                        new_pos += 1
                else:
                    new_pos = len(model)

                #FIXME: ADD QUICK DRAG DROP
                #print "URIS ", selection.data
                self.set_model(None)
                for uri in selection.get_uris():
                    uri = urllib.unquote(uri)
                    #print "AJOUT DE " , uri ," A LA POS = ", new_pos
                    if 0<=new_pos and new_pos<len(model):
                        iter = self.model_song.get_iter(new_pos)
                        self.add(uri,new_pos)
                        new_pos+=1
                    else:
                        self.add(uri)
                        new_pos+=1

                self.set_model(self.model_song)
                while gtk.events_pending():gtk.main_iteration()

        return True



    def add(self,uri,pos=False):
        #print "pos",pos
        song = DBManager.get_song_by_uri(uri)
        if song == None or song.is_stream:
            return False;

        if pos is False:
            self.model_song.append((song, song.get_int("tracknr"),song.sprint("title"),song.get_property("duration"),song.sprint("artist"),song.sprint("album")))
        else:
            self.model_song.insert(pos,(song, song.get_int("tracknr"),song.sprint("title"),song.get_property("duration"),song.sprint("artist"),song.sprint("album")))
        return True

    def add_song(self,input_song,pos=False):
        #print "pos",pos
        if input_song == None or input_song.is_stream:
            return False;

        #make a copie of song because this instance will be modified in place
        song = Song()
        song.read_from_song(input_song)

        if pos is False:
            self.model_song.append((song, song.get_int("tracknr"),song.sprint("title"),song.get_property("duration"),song.sprint("artist"),song.sprint("album")))
        else:
            self.model_song.insert(pos,(song, song.get_int("tracknr"),song.sprint("title"),song.get_property("duration"),song.sprint("artist"),song.sprint("album")))
        return

    def on_key_press(self, widget, event):
         if event.keyval == gtk.gdk.keyval_from_name("Delete"):
            self.remove_selected()
            return True

    def remove_selected(self,btn=None):
            model, rows = self.get_selection().get_selected_rows()
            rows.reverse()
            for row in rows:
                self.model_song.remove(self.model_song.get_iter(row))
            self.save_to_db()
            return True


    def get_menu(self):

        menu_ui = """
            <ui>
                <popup name="SongMenu">
                  <menuitem action="Remove"/>
                </popup>
            </ui>
            """

        self.uimanager = gtk.UIManager()
        self.uimanager.add_ui_from_string(menu_ui)
        accelgroup = self.uimanager.get_accel_group()

        actiongroup = gtk.ActionGroup('Listen')

        actiongroup.add_actions([('Remove', gtk.STOCK_DELETE, _('_Remove'), None,
                          _('Remove this song'), self.remove_selected)])





        self.uimanager.insert_action_group(actiongroup, 1)
        #gobject.idle_add(self.get_toplevel().add_accel_group,accelgroup)

        return self.uimanager.get_widget("/SongMenu")

